##############################################################################
#
# WatermatkImageField - Field that watermarks images for Archetypes
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
__author__  = ''' <nlaurance@zindep.com>'''
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.Field import ImageField
from Products.Archetypes.Widget import ImageWidget
from Products.Archetypes.Registry import registerField

from Products.CMFCore import CMFCorePermissions
from os import popen2
from cStringIO import StringIO

def pipe_command(command,value):
    """applies 'command' through popen to stdin 'value'
    return stdout"""
    (stdin,stdout,)=popen2(command)
    stdin.write(str(value))
    stdin.close()
    outdata=StringIO(stdout.read())
    stdout.close()
    return outdata

class WatermarkImageField(ImageField):
    """
    sends image uploaded to imagemagick for pre-treatment, especially watermarking.
    """
    _properties = ImageField._properties.copy()
    _properties.update({
        'watermark_command': None,
    })

    security = ClassSecurityInfo()

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        if not value:
            return
        
        # Do we have to delete the image?
        if value=="DELETE_IMAGE":
            self.removeScales(instance, **kwargs)
            # unset main field too
            ObjectField.unset(self, instance, **kwargs)
            return

        kwargs.setdefault('mimetype', None)
        default = self.getDefault(instance)
        value, mimetype, filename = self._process_input(value, default=default,
                                                        instance=instance, **kwargs)
        # value is an OFS.Image.File based instance
        # don't store empty images
        size = getattr(value, 'size', None)
        if size == 0:
            return
        
        kwargs['mimetype'] = mimetype
        kwargs['filename'] = filename

        try:
            data = self.rescaleOriginal(value, **kwargs)
        except ConflictError:
            raise
        except:
            if not self.swallowResizeExceptions:
                raise
            else:
                log_exc()
                data = str(value.data)
        # the trick is here
        imgdata=pipe_command(self.watermark_command,data).getvalue()
        self.createOriginal(instance, imgdata, **kwargs)
        self.createScales(instance, value=imgdata)


registerField(WatermarkImageField,
              title='Watermark Image',
              description='A field that watermarks images.')
